@@ -2,6 +2,7 @@ import asyncio
 import importlib
 import inspect
 import re
+import sys
 from collections.abc import AsyncGenerator
 from functools import cached_property
 from pathlib import Path
@@ -21,8 +22,10 @@ from pydantic import (
     PrivateAttr,
     SkipValidation,
     computed_field,
+    model_validator,
 )
 
+from ..errors import InterruptError, bail
 from ..state import get_run_context
 from ..types.content import ToolUseContent
 from ..types.events import OutputEvent
@@ -37,19 +40,57 @@ logger = structlog.get_logger("timbal.core.agent")
 SYSTEM_PROMPT_FN_PATTERN = re.compile(r"\{[a-zA-Z0-9_]*::[a-zA-Z0-9_]+(?:::[a-zA-Z0-9_]+)*\}")
 
 
-# TODO Add more params here
 class AgentParams(BaseModel):
     """Parameter model for Agent execution.
     
     Defines the input parameters that agents accept when called.
-    Supports flexible system prompt composition through templates.
+    Use either 'prompt' or 'messages', not both:
+    - 'prompt': Single message input. The framework will automatically resolve 
+      and include memory from previous runs.
+    - 'messages': Explicit list of messages. No automatic memory resolution occurs;
+      you have full control over the message history.
     """
     model_config = ConfigDict(extra="allow")
 
-    prompt: Message = Field(
-        ...,
-        description="Input message to send to the agent.",
+    prompt: Message | None = Field(
+        None,
+        description="Single input message. Framework automatically resolves memory from previous runs.",
     )
+    messages: list[Message] | None = Field(
+        None,
+        description="Explicit list of messages. No automatic memory resolution; full manual control.",
+    )
+
+    @model_validator(mode="after")
+    def validate_prompt_or_messages(self) -> "AgentParams":
+        """Ensure exactly one of prompt or messages is set."""
+        if self.prompt is not None and self.messages is not None:
+            logger.warning("Calling agent with both 'prompt' and 'messages'. Using 'messages'.")
+        if self.prompt is None and self.messages is None:
+            raise ValueError("Must specify either 'prompt' or 'messages'.")
+        return self
+
+    @classmethod
+    def model_json_schema(cls, **kwargs: Any) -> dict[str, Any]:
+        """Override model_json_schema to return a custom schema so that agents can be used as tools more easily."""
+        return {
+            "type": "object",
+            "properties": {"prompt": {
+                "type": "object",
+                "title": "TimbalMessage",
+                "properties": {
+                    "role": {
+                        "type": "string",
+                        "enum": ["user"],
+                    },
+                    "content": {
+                        "type": "array",
+                        "items": {},
+                    }
+                },
+            }},
+            "required": ["prompt"],
+        }
 
 
 class Agent(Runnable):
@@ -114,32 +155,73 @@ class Agent(Runnable):
                 assert len(path_parts) >= 2, f"Invalid path format for system prompt: {path}. Review the SYSTEM_PROMPT_FN_PATTERN regex."
                 module, fn_i = None, None
                 if path_parts[0] == "":
+                    # File-relative import: look in caller's globals first
                     frame = inspect.currentframe()
-                    caller_file = None
+                    caller_globals = None
                     while frame:
                         frame = frame.f_back
                         if frame is None:
                             break
-                        frame_file = frame.f_globals.get("__file__", "")
                         frame_self = frame.f_locals.get("self")
                         if not isinstance(frame_self, Agent):
-                            caller_file = Path(frame_file)
+                            caller_globals = frame.f_globals
                             break
-                    assert caller_file is not None, "Could not determine caller file for Agent constructor."
-                    agent_path = Path(caller_file).expanduser().resolve()
-                    for fn_i in range(1, len(path_parts)):
-                        module_path = agent_path / "/".join(path_parts[:-fn_i])
-                        try:
-                            module_spec = importlib.util.spec_from_file_location(module_path.stem, module_path.as_posix())
-                            if not module_spec or not module_spec.loader:
-                                raise ValueError(f"Failed to load module {module_path}")
-                            module = importlib.util.module_from_spec(module_spec)
-                            module_spec.loader.exec_module(module)
-                            logger.info(f"Loaded module '{module_path}' for system prompt callable '{text}'")
-                            break
-                        except Exception:
-                            pass
+                    assert caller_globals is not None, "Could not determine caller globals for Agent constructor."
+                    
+                    # Try to resolve from caller's globals directly (handles same-file definitions)
+                    fn = caller_globals
+                    try:
+                        for attr_name in path_parts[1:]:
+                            fn = fn[attr_name] if isinstance(fn, dict) else getattr(fn, attr_name)
+                        logger.info(f"Resolved callable '{text}' from caller's globals")
+                    except (KeyError, AttributeError):
+                        # If not in globals, try loading the module
+                        caller_file = Path(caller_globals.get("__file__", "")).expanduser().resolve()
+                        agent_path = caller_file
+                        for fn_i in range(1, len(path_parts)):
+                            module_path = agent_path / "/".join(path_parts[:-fn_i])
+                            try:
+                                # Use absolute path as module identifier
+                                module_path_str = str(module_path.resolve())
+                                # Check if module is already loaded in sys.modules by absolute path
+                                if module_path_str in sys.modules:
+                                    module = sys.modules[module_path_str]
+                                    logger.info(f"Using already loaded module '{module_path}' for system prompt callable '{text}'")
+                                    break
+                                # Load the module if not already loaded
+                                module_spec = importlib.util.spec_from_file_location(module_path_str, module_path.as_posix())
+                                if not module_spec or not module_spec.loader:
+                                    raise ValueError(f"Failed to load module {module_path}")
+                                module = importlib.util.module_from_spec(module_spec)
+                                # Register in sys.modules BEFORE executing to prevent re-entry
+                                sys.modules[module_path_str] = module
+                                module_spec.loader.exec_module(module)
+                                logger.info(f"Loaded module '{module_path}' for system prompt callable '{text}'")
+                                break
+                            except Exception:
+                                pass
+                        else:
+                            for fn_i in range(1, len(path_parts)):
+                                module_path = ".".join(path_parts[:-fn_i])
+                                try:
+                                    module = importlib.import_module(module_path)
+                                    logger.info(f"Loaded module '{module_path}' for system prompt callable '{text}'")
+                                    break
+                                except Exception:
+                                    pass
+                        fn = module
+                        for j in path_parts[-fn_i:]:
+                            fn = getattr(fn, j)
+                    
+                    inspect_result = self._inspect_callable(fn)
+                    self._system_prompt_callables[text] = {
+                        "start": match.start(),
+                        "end": match.end(),
+                        "callable": fn,
+                        "is_coroutine": inspect_result["is_coroutine"],
+                    }
                 else:
+                    # Package import (e.g., {os::getcwd})
                     for fn_i in range(1, len(path_parts)):
                         module_path = ".".join(path_parts[:-fn_i])
                         try:
@@ -148,18 +230,18 @@ class Agent(Runnable):
                             break
                         except Exception:
                             pass
-                fn = module
-                for j in path_parts[-fn_i:]:
-                    fn = getattr(fn, j)
-                inspect_result = self._inspect_callable(fn)
-                self._system_prompt_callables[text] = {
-                    "start": match.start(),
-                    "end": match.end(),
-                    "callable": fn,
-                    "is_coroutine": inspect_result["is_coroutine"],
-                }
+                    fn = module
+                    for j in path_parts[-fn_i:]:
+                        fn = getattr(fn, j)
+                    inspect_result = self._inspect_callable(fn)
+                    self._system_prompt_callables[text] = {
+                        "start": match.start(),
+                        "end": match.end(),
+                        "callable": fn,
+                        "is_coroutine": inspect_result["is_coroutine"],
+                    }
         
-        model_provider, model_name = self.model.split("/")
+        model_provider, model_name = self.model.split("/", 1)
         if model_provider == "anthropic":
             if not self.model_params.get("max_tokens"):
                 raise ValueError("'max_tokens' is required for claude models.")
@@ -292,7 +374,9 @@ class Agent(Runnable):
                 # Extract conversation history from parent's LLM calls
                 # TODO: Handle multiple call_ids for this subagent
                 llm_spans = parent_trace.get_path(self._llm._path)
-                assert len(llm_spans) >= 1, f"Agent trace does not have any records for path {self._llm._path}"
+                # In a subagent, this can be empty if the parent agent didn't call the LLM
+                if not len(llm_spans):
+                    return memory
                 # Get the most recent LLM interaction
                 llm_input_messages = llm_spans[-1].input.get("messages", []) # TODO Put an assertion in here
                 llm_output_message = llm_spans[-1].output
@@ -301,56 +385,74 @@ class Agent(Runnable):
                     *[Message.validate(m) for m in llm_input_messages], 
                     Message.validate(llm_output_message)
                 ]
+                for content in memory[-1].content:
+                    if content.type != "tool_use":
+                        continue
+                    tool_result_path = f"{self._path}.{content.name}"
+                    tool_result_spans = parent_trace.get_path(tool_result_path)
+                    assert len(tool_result_spans) >= 1, f"Agent trace does not have any records for path {tool_result_path}"
+                    for tool_result_span in tool_result_spans:
+                        if tool_result_span.metadata.get("tool_call_id") == content.id:
+                            tool_result_content = tool_result_span.output.content if isinstance(tool_result_span.output, Message) else tool_result_span.output
+                            if tool_result_span.status["code"] == "cancelled" and tool_result_span.status["reason"] == "interrupted":
+                                tool_result_content = "</interrupted>"
+                            tool_result_message = Message.validate({
+                                "role": "tool",
+                                "content": [{
+                                    "type": "tool_result",
+                                    "id": content.id,
+                                    "content": tool_result_content,
+                                }]
+                            })
+                            memory.append(tool_result_message)
         return memory
 
-    
-    async def _enqueue_tool_events(self, tool_call: ToolUseContent, queue: asyncio.Queue) -> None:
-        """Execute a single tool call and enqueue its events.
-        
-        This method runs a tool call asynchronously and puts all generated
-        events into a shared queue for multiplexed processing. Used by
-        _multiplex_tools to handle concurrent tool execution.
-        
-        Args:
-            tool_call: The tool call content with name and input parameters
-            queue: Async queue to put events into
-        """
-        tool = self._tools_by_name[tool_call.name]
-        # Execute tool and enqueue all events
-        async for event in tool(**tool_call.input):
-            await queue.put((tool_call, event))
-        # Signal completion with a sentinel value (None)
-        await queue.put((tool_call, None))
-
 
     async def _multiplex_tools(self, tool_calls: list[ToolUseContent]) -> AsyncGenerator[Any, None]:
-        """Execute multiple tool calls concurrently and multiplex their events.
-        
-        This method enables concurrent execution of multiple tool calls requested
-        by the LLM, significantly improving performance when tools can run in parallel.
-        Events from all tools are multiplexed and yielded as they become available.
-        
-        Args:
-            tool_calls: List of tool calls to execute concurrently
-            
-        Yields:
-            Tuples of (tool_call, event) as events are generated by tools
-        """
+        """Execute multiple tool calls concurrently and multiplex their events."""
         queue = asyncio.Queue()
-        # Start all tool executions concurrently
-        tasks = [asyncio.create_task(self._enqueue_tool_events(tc, queue)) for tc in tool_calls]
-        # Process events as they arrive from any tool
-        remaining = len(tasks)
-        while remaining > 0:
-            tool_call, event = await queue.get()
-            if event is None:
-                # Sentinel value indicates a tool completed
-                remaining -= 1
-            else:
-                # Yield event along with which tool call generated it
-                yield tool_call, event
-        # Ensure all tasks complete cleanly
-        await asyncio.gather(*tasks)
+        tasks = []
+        
+        async def consume_tool(tool_call: ToolUseContent):
+            """Consume events from a single tool and put them in the queue."""
+            tool = self._tools_by_name[tool_call.name]
+            try:
+                async for event in tool(**tool_call.input):
+                    # We need to link the tool call id to the span so that we can later match when resolving memory
+                    if event.type == "START":
+                        tool_call_id = event.call_id 
+                        tool_call_span = get_run_context()._trace[tool_call_id]
+                        tool_call_span.metadata["tool_call_id"] = tool_call.id
+                    await queue.put((tool_call, event))
+            finally:
+                await queue.put((tool_call, None))  # Sentinel
+        
+        try:
+            # Start all tool tasks
+            for tc in tool_calls:
+                task = asyncio.create_task(consume_tool(tc))
+                tasks.append(task)
+            
+            # Consume events as they arrive
+            remaining = len(tool_calls)
+            while remaining > 0:
+                tool_call, event = await queue.get()
+                if event is None:
+                    remaining -= 1
+                else:
+                    yield tool_call, event
+        except (asyncio.CancelledError, GeneratorExit, InterruptError):
+            # Cancellation or generator closed - clean up gracefully
+            raise
+        finally:
+            # Cancel all pending tasks
+            for task in tasks:
+                if not task.done():
+                    task.cancel()
+            
+            # Wait for all cancellations to complete, suppressing errors
+            if tasks:
+                await asyncio.gather(*tasks, return_exceptions=True)
 
 
     async def handler(self, **kwargs: Any) -> AsyncGenerator[Any, None]:
@@ -377,9 +479,13 @@ class Agent(Runnable):
         system_prompt = kwargs.pop("system_prompt", None)
         if not system_prompt:
             system_prompt = await self._resolve_system_prompt()
-        # Initialize conversation with memory + user prompt
-        messages = await self._resolve_memory()
-        messages.append(kwargs.pop("prompt"))
+            
+        # We allow the user to pass a 'hardcoded' list of messages
+        # Or simply a prompt and we attempt to resolve memory
+        messages = kwargs.pop("messages", [])
+        if not messages:
+            messages = await self._resolve_memory()
+            messages.append(kwargs.pop("prompt"))
 
         i = 0
         while True:
@@ -396,9 +502,13 @@ class Agent(Runnable):
                     # If the LLM call fails, we want to propagate the error upwards
                     if event.error is not None:
                         raise RuntimeError(event.error)
+                    # If the LLM was interrupted, propagate the interruption
+                    if event.status.code == "cancelled" and event.status.reason == "interrupted":
+                        raise InterruptError(event.call_id)
                     assert isinstance(event.output, Message), f"Expected event.output to be a Message, got {type(event.output)}"
                     # Add LLM response to conversation for next iteration
                     messages = messages + [event.output]
+                    
                 yield event
             
             tool_calls = [
@@ -412,6 +522,9 @@ class Agent(Runnable):
             async for tool_call, event in self._multiplex_tools(tool_calls):
                 # Only process events from immediate children (not nested subagents)
                 if isinstance(event, OutputEvent) and event.path.count(".") == self._path.count(".") + 1:
+                    # Propagate bail
+                    if event.status.code == "cancelled" and event.status.reason == "early_exit":
+                        bail(event.status.message)
                     message = Message.validate({
                         "role": "tool",
                         "content": [{
@@ -422,4 +535,4 @@ class Agent(Runnable):
                     })
                     messages.append(message)
                 yield event
-            i += 1
\ No newline at end of file
+            i += 1
