@@ -25,7 +25,7 @@ from uuid_extensions import uuid7
 
 from ..collectors import get_collector_registry
 from ..collectors.impl.timbal import TimbalCollector
-from ..errors import EarlyExit
+from ..errors import EarlyExit, InterruptError
 from ..state import (
     get_or_create_run_context,
     get_parent_call_id,
@@ -357,11 +357,19 @@ class Runnable(ABC, BaseModel):
             selected_params.difference_update(self.schema_exclude_params)
 
         # Filter properties to only include selected params
-        properties = {
-            k: v 
-            for k, v in self.params_model_schema["properties"].items()
-            if k in selected_params
-        }
+        properties = {}
+        for k, v in self.params_model_schema["properties"].items():
+            if k not in selected_params:
+                continue
+            # Simplify the structure of messages for LLM tool calling
+            if v.get("title") == "TimbalMessage":
+                properties[k] = {
+                    "type": "string",
+                    "description": "The input message to the agent.",
+                }
+            else:
+                properties[k] = v
+
         return {
             **self.params_model_schema,
             "properties": properties,
@@ -529,6 +537,7 @@ class Runnable(ABC, BaseModel):
         span.input = input
         span._input_dump = None # ? await dump(input)
         span._output_dump = None
+        collector = None
         try:
             start_event = StartEvent(
                 run_id=run_context.id,
@@ -605,17 +614,18 @@ class Runnable(ABC, BaseModel):
                     # Keep the final result
                     output = collector.result()
 
-            # If the output is an OutputEvent, we extract the output
-            # to avoid nesting an output event inside another output event
-            if isinstance(output, OutputEvent):
-                output = output.output
-            span.output = output
-            span._output_dump = await dump(output)
             span.status = RunStatus(
                 code="success",
                 reason=None, # TODO
                 message=None # TODO
             )
+            # If the output is an OutputEvent, we extract the output
+            # to avoid nesting an output event inside another output event
+            if isinstance(output, OutputEvent):
+                if output.status.code == "cancelled":
+                    span.status = output.status
+                output = output.output
+            span.output = output
             
             # Restore the call context to this runnable before executing post_hook
             # This ensures post_hook modifies the correct span, not any nested ones
@@ -624,6 +634,9 @@ class Runnable(ABC, BaseModel):
                 set_parent_call_id(_call_id)
             if self.post_hook is not None:
                 await self._execute_runtime_callable(self.post_hook, self._post_hook_is_coroutine)
+
+            # Post hook might modify the output, so we dump afterwards 
+            span._output_dump = await dump(output)
             
         except EarlyExit as early_exit:
             span.status = RunStatus(
@@ -631,6 +644,21 @@ class Runnable(ABC, BaseModel):
                 reason="early_exit",
                 message=str(early_exit)
             )
+            span.output = None
+            span._output_dump = None
+
+        except (asyncio.CancelledError, InterruptError) as e:
+            logger.warning("Interrupted", run_id=run_context.id, call_id=span.call_id)
+            # Create a message with collected chunks or cancellation info
+            if collector is not None:
+                span.output = collector.result()
+                span._output_dump = await dump(span.output)
+            span.status = RunStatus(
+                code="cancelled",
+                reason="interrupted",
+                message=str(e)
+            )
+            yield InterruptError(_call_id)
 
         except Exception as err:
             error = {
