@@ -1,38 +1,6 @@
-"""
-This module provides a File class that wraps various types of file sources,
-including local files, URLs, enabling uniform interaction through
-a file-like interface.
-
-It is inspired by and based on the design found in the Replicate Cog project.
-See original implementation at:
-https://github.com/replicate/cog/blob/main/python/cog/types.py
-
-Usage:
-
-1. Validating and opening a local file:
-   >>> file_instance = File.validate('path/to/local/file.txt')
-   >>> print(file_instance.read())
-
-2. Validating and opening a data url:
-   >>> file_instance = File.validate('data:text/plain;base64,SGVsbG8sIFdvcmxkIQ%3D%3D')
-   >>> print(file_instance.read()) # b'Hello, World!'
-
-3. Opening a file from a URL:
-   >>> from PIL import Image
-   >>> file_instance = File.validate('https://example.com/image.png')
-   >>> image = Image.open(file_instance)
-   >>> image.show()
-
-4. Opening a file-like object:
-   >>> from io import BytesIO
-   >>> file_instance = File.validate(BytesIO(b'Hello, World!'))
-   >>> print(file_instance.read())
-"""
-
 import base64
 import io
 import mimetypes
-import os
 import tempfile
 from collections.abc import Callable, Iterator
 from pathlib import Path
@@ -51,8 +19,8 @@ from pydantic import (
 from pydantic_core import CoreSchema, core_schema
 from uuid_extensions import uuid7
 
-from ..state import get_run_context
-from ..utils import _platform_api_call
+from ..platform.utils import _request
+from ..state import get_or_create_run_context
 
 
 class File(io.IOBase):
@@ -82,10 +50,6 @@ class File(io.IOBase):
         "__content_type__",
     )
 
-    __max_file_size__: int | None = (
-        int(os.getenv("TIMBAL_MAX_FILE_SIZE")) if os.getenv("TIMBAL_MAX_FILE_SIZE") else None
-    )
-    """ Maximum allowed file size in bytes"""
 
 
     def __init__(
@@ -109,14 +73,20 @@ class File(io.IOBase):
         if source_scheme == "bytes":
             if self.__source_extension__:
                 content_type, _ = mimetypes.guess_type(f"tmp{self.__source_extension__}")
-                if content_type is None:
-                    content_type = f"timbal/{self.__source_extension__}"
+                # Python 3.11 doesn't recognize .md files, add fallback
+                if content_type is None and self.__source_extension__ == ".md":
+                    content_type = "text/markdown"
+                elif content_type is None:
+                    content_type = "application/octet-stream"
             else:
                 content_type = "application/octet-stream"
         else:
             content_type, _ = mimetypes.guess_type(str(self))
-            if content_type is None:
-                content_type = f"timbal/{self.__source_extension__}"
+            # Python 3.11 doesn't recognize .md files, add fallback
+            if content_type is None and str(self).endswith(".md"):
+                content_type = "text/markdown"
+            elif content_type is None:
+                content_type = "application/octet-stream"
         object.__setattr__(self, "__content_type__", content_type)
 
     
@@ -200,6 +170,8 @@ class File(io.IOBase):
                 fileext = object.__getattribute__(self, "__source_extension__")
                 if fileext:
                     fileobj.name = f"{uuid7()}{fileext}"
+            if hasattr(fileobj, "__content_type__"):
+                object.__setattr__(self, "__content_type__", fileobj.__content_type__)
             object.__setattr__(self, "__fileobj__", fileobj)
         return fileobj
 
@@ -280,10 +252,6 @@ class File(io.IOBase):
             path = Path(value).expanduser().resolve()
             if not path.exists() or not path.is_file():
                 raise ValueError("Invalid file path.")
-            if cls.__max_file_size__ is not None:
-                file_size = path.stat().st_size  # bytes
-                if file_size > cls.__max_file_size__:
-                    raise ValueError(f"File size exceeds the maximum allowed size of {cls.__max_file_size__} bytes.")
             parsed_url_extension = path.suffix
             return File(
                 value,
@@ -294,16 +262,6 @@ class File(io.IOBase):
 
         # Data urls: data:[<mediatype>][;base64],<data>.
         if parsed_url.scheme == "data":
-            if cls.__max_file_size__ is not None:
-                comma_idx = value.find(",")
-                # Redundant check.
-                if comma_idx == -1:
-                    raise ValueError("Invalid data URL.")
-                encoded = value[comma_idx + 1 :]
-                # ? We could count the padding if we want to be more precise.
-                estimated_size = len(encoded) * 3 // 4
-                if estimated_size > cls.__max_file_size__:
-                    raise ValueError(f"File size exceeds the maximum allowed size of {cls.__max_file_size__} bytes.")
             parsed_url_mimetype = parsed_url.path.split(";", 1)
             if len(parsed_url_mimetype) != 2:
                 raise ValueError("Data url must specify a mimetype.")
@@ -322,11 +280,6 @@ class File(io.IOBase):
             )
 
         if parsed_url.scheme == "http" or parsed_url.scheme == "https":
-            response = requests.head(value)
-            if cls.__max_file_size__ is not None:
-                content_length = int(response.headers.get("Content-Length", 0))
-                if content_length > cls.__max_file_size__:
-                    raise ValueError(f"File size exceeds the maximum allowed size of {cls.__max_file_size__} bytes.")
             parsed_url_name = parsed_url.path.split("/")[-1]
             parsed_url_extension = f".{parsed_url_name.split('.')[-1]}" if "." in parsed_url_name else ""
             return File(
@@ -367,24 +320,24 @@ class File(io.IOBase):
         # ? Add more resource specifiers
     ) -> str | None:
         """Persist the file to some storage.
-        If there's no context or no timbal_platform_config, the file will be persisted to local disk.
+        If there's no run context or valid platform config, the file will be persisted to local disk.
         If there's a platform configuration, the file will be uploaded to the platform.
         If the file is already persisted, it will be returned as is.
         """
         if self.__persisted__ is not None:
             return self.__persisted__
 
-        context = get_run_context()
+        run_context = get_or_create_run_context()
 
         if self.__source_scheme__ == "url":
             url = str(self)
-            if not context or not context.timbal_platform_config:
+            if not run_context.platform_config:
                 return url
-            elif url.startswith(f"https://{context.timbal_platform_config.cdn}"):
+            elif url.startswith(f"https://{run_context.platform_config.cdn}"):
                 object.__setattr__(self, "__persisted__", url)
                 return url
 
-        if not context or not context.timbal_platform_config:
+        if not run_context.platform_config or (not org_id and not run_context.platform_config.subject):
             if self.__source_scheme__ == "local_path":
                 local_path = str(self)
                 object.__setattr__(self, "__persisted__", local_path)
@@ -400,7 +353,7 @@ class File(io.IOBase):
             object.__setattr__(self, "__persisted__", temp_path)
             return temp_path
 
-        subject = context.timbal_platform_config.subject
+        subject = run_context.platform_config.subject
         org_id = org_id or subject.org_id
         # ? We could add more subject info here
 
@@ -410,9 +363,10 @@ class File(io.IOBase):
         path = f"orgs/{org_id}/files"
         files = {"file": (self.name, content, self.__content_type__)}
 
-        res = await _platform_api_call("POST", path, files=files)
+        res = await _request("POST", path, files=files)
+        res_json = res.json()
         # ? We could use an UploadFileResponse pydantic model
-        url = res.json()["url"]
+        url = res_json["url"]
         object.__setattr__(self, "__persisted__", url)
         return url
 
@@ -421,8 +375,8 @@ class File(io.IOBase):
     def serialize(
         cls, 
         value: Any, 
-        *args,
-        **kwargs,
+        *args,  # noqa: ARG003
+        **kwargs,  # noqa: ARG003
     ) -> str:
         """Serialize the file to a data url string. Bytes-like files are not supported will have octet-stream as content type."""
         # When creating a model with fields with File type that are nullable,
@@ -463,7 +417,26 @@ class File(io.IOBase):
         res = requests.get(url, stream=True)
         res.raise_for_status()
         fileobj = io.BytesIO(res.content)
-        fileobj.name = url.split("/")[-1]
+        # Try to get a meaningful filename from URL or Content-Disposition header
+        filename = None
+        # Check Content-Disposition header first
+        content_disposition = res.headers.get("Content-Disposition")
+        if content_disposition and "filename=" in content_disposition:
+            try:
+                filename = content_disposition.split("filename=")[1].strip('"\'')
+            except (IndexError, AttributeError):
+                pass
+        # Fallback to URL path
+        if not filename:
+            filename = url.split("/")[-1]
+        fileobj.name = filename
+        # Set content type from server response
+        # NOTE: This will overwrite the File's extension-based content type detection
+        server_content_type = res.headers.get("Content-Type", "application/octet-stream")
+        # Only set __content_type__ if server provides a specific type (not generic binary)
+        if server_content_type not in ("application/octet-stream", "binary/octet-stream"):
+            fileobj.__content_type__ = server_content_type
+        # For generic types, let the File class keep its extension-based detection
         return fileobj
 
 
